package app.mar.game;

import android.app.Activity;
import android.app.AlertDialog;
import android.content.DialogInterface;
import android.graphics.Bitmap;
import android.util.Log;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import java.io.Serializable;
import java.util.ArrayList;

import app.mar.exceptions.BadConfigFileException;
import app.mar.exceptions.BadResourceConfigException;
import app.mar.exceptions.ResourceFileNotFoundException;
import app.mar.exceptions.ResourceNotFoundException;
import app.mar.utils.files.JSONAssetsManager;
/**
 * Created by ptitcois on 19/08/16.
 */
public class Resource implements Serializable {

    protected String mName;
    protected String mComment = "null";
    protected boolean isLoaded = false;
    protected boolean mIsDisplay = false;
    protected int   mStage;
    protected String mType = ""; //3D Image Audio Video
    protected String mTitle= ""; //3D Image Audio Video
    private  static JSONObject mRessourcesList = null;
    public static ArrayList<String> mNotFound = new ArrayList<>();
    protected String mFile = null;

    public String getFile() {
        if(mFile!=null)
            return mFile;
        return mName;
    }

    public boolean isFile() {
        return mFile!=null;
    }

    public static void initResourceList(Activity act)
    {
        if(mRessourcesList!=null) return;
        Game.logv( "Chargement de la liste des ressources...");
        JSONObject obj = JSONAssetsManager.load(act, "ressources.res");
        if(obj==null)
        {
            Game.logw("Attention il n'existe pas de fichier pour la liste des ressrouecs");
            return;
        }
        mRessourcesList=obj;
    }

    public static JSONObject getRessourceDataFromList(String name)
    {
        try {
            if(mRessourcesList!=null)
                return mRessourcesList.getJSONObject(name);
        } catch (JSONException e) {
            Game.logw("Impossiblle de trouver la ressource '"+name+"' dans la liste des ressources");
        }
        return null;
    }

    public static void logNotFound()
    {
        Log.e("LoadingRes", "Liste des ressources non-trouvées:");
        for(int i=0; i<mNotFound.size(); i++)
            Log.e("LoadingRes", "\t'"+mNotFound.get(i)+"'");
    }

    public static void setNotFound(String name)
    {
        for(int i=0; i< mNotFound.size(); i++)
            if(mNotFound.get(i).compareTo(name)==0) return;
        mNotFound.add(name);
    }

    protected void loadResourceData(Activity act)
    {
        initResourceList(act);
        Game.logv( "--- Chargement de la ressource '"+mName+"' --- ");
        JSONObject obj = JSONAssetsManager.load(act, mName+".res");
        if(obj==null) obj = getRessourceDataFromList(mName);
        if(obj == null)
        {
            Game.loge("\tLa ressource '"+mName+"' est introuvable.");
            setNotFound(mName);
            Game.fail();
            throw new ResourceNotFoundException(mName);
        }

        //mandatory
        try {
            mType = obj.getString("type");
        } catch(JSONException e)
        {
            Game.loge("\tLa ressource '"+mName+"' n'a pas d'attribut 'type' valide (String)");
            e.printStackTrace();
            throw new BadResourceConfigException("Bad parameter 'type' ");
        }

        try {
            mTitle = obj.getString("title");
        } catch (JSONException e) {
        }

        try {
            mComment=obj.getString("comment");
        } catch (JSONException e) {
        }
        try {
            mStage = obj.getInt("stage");
            isLoaded=true;
        } catch (JSONException e) {
        }

        try{
            mFile = obj.getString("file");
        }catch (JSONException e)
        {
            mFile=null;
        }

        try {
            mIsDisplay=(boolean) obj.getBoolean("display");
        } catch (JSONException e) {
            mIsDisplay=false;
        }
    }

    public boolean hasFile(){
        return mFile!=null;
    }


    public Resource(String name, Activity act)
    {
        mName=name;
        loadResourceData(act);
    }


    public String getName() {
        return mName;
    }

    public String getType() {
        return mType;
    }

    public String getTitle() {
        return mTitle;
    }


    public String getComment() {
        return mComment;
    }

    public int getStage() {
        return mStage;
    }

    public  JSONObject getJson()
    {
        JSONObject jo = new JSONObject();
        try {
            jo.put("name", mName);
            jo.put("stage", mStage);
        } catch (JSONException e) {
            e.printStackTrace();
        }
        return jo;
    }


    public String toString()
    {
        return mTitle;
    }

    public static final String IMAGE = "Image";
    public static final String VIDEO = "Video";
    public static final String AUDIO = "Audio";


    public boolean isAudio()
    {
        return mType.toLowerCase().compareTo(AUDIO.toLowerCase())==0;
    }

    public boolean isVideo()
    {
        return mType.toLowerCase().compareTo(VIDEO.toLowerCase())==0;
    }

    public boolean isImage()
    {
        return mType.toLowerCase().compareTo(IMAGE.toLowerCase())==0;
    }

    public boolean isDisplay() {
        return mIsDisplay;
    }
}
